@@ -1,7 +1,7 @@
 module("selector");
 
 test("element", function() {
-	expect(18);
+	expect(19);
 	reset();
 
 	ok( jQuery("*").size() >= 30, "Select all" );
@@ -16,20 +16,22 @@ test("element", function() {
 	t( "Parent Element", "div p", ["firstp","ap","sndp","en","sap","first"] );
 	equals( jQuery("param", "#object1").length, 2, "Object/param as context" );
 
-	isSet( jQuery("p", document.getElementsByTagName("div")), q("firstp","ap","sndp","en","sap","first"), "Finding elements with a context." );
-	isSet( jQuery("p", "div"), q("firstp","ap","sndp","en","sap","first"), "Finding elements with a context." );
-	isSet( jQuery("p", jQuery("div")), q("firstp","ap","sndp","en","sap","first"), "Finding elements with a context." );
-	isSet( jQuery("div").find("p"), q("firstp","ap","sndp","en","sap","first"), "Finding elements with a context." );
+	same( jQuery("p", document.getElementsByTagName("div")).get(), q("firstp","ap","sndp","en","sap","first"), "Finding elements with a context." );
+	same( jQuery("p", "div").get(), q("firstp","ap","sndp","en","sap","first"), "Finding elements with a context." );
+	same( jQuery("p", jQuery("div")).get(), q("firstp","ap","sndp","en","sap","first"), "Finding elements with a context." );
+	same( jQuery("div").find("p").get(), q("firstp","ap","sndp","en","sap","first"), "Finding elements with a context." );
+
+	same( jQuery("#form").find("select").get(), q("select1","select2","select3"), "Finding selects with a context." );
 	
 	ok( jQuery("#length").length, '&lt;input name="length"&gt; cannot be found under IE, see #945' );
 	ok( jQuery("#lengthtest input").length, '&lt;input name="length"&gt; cannot be found under IE, see #945' );
 
 	// Check for unique-ness and sort order
-	isSet( jQuery("*"), jQuery("*, *"), "Check for duplicates: *, *" );
-	isSet( jQuery("p"), jQuery("p, div p"), "Check for duplicates: p, div p" );
+	same( jQuery("*").get(), jQuery("*, *").get(), "Check for duplicates: *, *" );
+	same( jQuery("p").get(), jQuery("p, div p").get(), "Check for duplicates: p, div p" );
 
-	t( "Checking sort order", "h2, h1", ["header", "banner", "userAgent"] );
-	t( "Checking sort order", "h2:first, h1:first", ["header", "banner"] );
+	t( "Checking sort order", "h2, h1", ["qunit-header", "qunit-banner", "qunit-userAgent"] );
+	t( "Checking sort order", "h2:first, h1:first", ["qunit-header", "qunit-banner"] );
 	t( "Checking sort order", "p, p a", ["firstp", "simon1", "ap", "google", "groups", "anchor1", "mark", "sndp", "en", "yahoo", "sap", "anchor2", "simon", "first"] );
 });
 
@@ -51,10 +53,11 @@ if ( location.protocol != "file:" ) {
 }
 
 test("broken", function() {
-	expect(7);
+	expect(8);
 	function broken(name, selector) {
 		try {
 			jQuery(selector);
+			ok( false, name + ": " + selector );
 		} catch(e){
 			ok(  typeof e === "string" && e.indexOf("Syntax error") >= 0,
 				name + ": " + selector );
@@ -68,6 +71,7 @@ test("broken", function() {
 	broken( "Broken Selector", "()", [] );
 	broken( "Broken Selector", "<>", [] );
 	broken( "Broken Selector", "{}", [] );
+	broken( "Doesn't exist", ":visble", [] );
 });
 
 test("id", function() {
@@ -96,14 +100,16 @@ test("id", function() {
 	t( "All Children of ID", "#foo > *", ["sndp", "en", "sap"] );
 	t( "All Children of ID with no children", "#firstUL > *", [] );
 	
-	jQuery('<a name="tName1">tName1 A</a><a name="tName2">tName2 A</a><div id="tName1">tName1 Div</div>').appendTo('#main');
+	var a = jQuery('<div><a name="tName1">tName1 A</a><a name="tName2">tName2 A</a><div id="tName1">tName1 Div</div></div>').appendTo('#main');
 	equals( jQuery("#tName1")[0].id, 'tName1', "ID selector with same value for a name attribute" );
 	equals( jQuery("#tName2").length, 0, "ID selector non-existing but name attribute on an A tag" );
+	a.remove();
+
 	t( "ID Selector on Form with an input that has a name of 'id'", "#lengthtest", ["lengthtest"] );
 	
 	t( "ID selector with non-existant ancestor", "#asdfasdf #foobar", [] ); // bug #986
 
-	isSet( jQuery("body").find("div#form"), [], "ID selector within the context of another element" );
+	same( jQuery("body").find("div#form").get(), [], "ID selector within the context of another element" );
 
 	t( "Underscore ID", "#types_all", ["types_all"] );
 	t( "Dash ID", "#fx-queue", ["fx-queue"] );
@@ -119,10 +125,10 @@ test("class", function() {
 	t( "Class Selector w/ Element", "a.blog", ["mark","simon"] );
 	t( "Parent Class Selector", "p .blog", ["mark","simon"] );
 
-	isSet( jQuery(".blog", document.getElementsByTagName("p")), q("mark", "simon"), "Finding elements with a context." );
-	isSet( jQuery(".blog", "p"), q("mark", "simon"), "Finding elements with a context." );
-	isSet( jQuery(".blog", jQuery("p")), q("mark", "simon"), "Finding elements with a context." );
-	isSet( jQuery("p").find(".blog"), q("mark", "simon"), "Finding elements with a context." );
+	same( jQuery(".blog", document.getElementsByTagName("p")).get(), q("mark", "simon"), "Finding elements with a context." );
+	same( jQuery(".blog", "p").get(), q("mark", "simon"), "Finding elements with a context." );
+	same( jQuery(".blog", jQuery("p")).get(), q("mark", "simon"), "Finding elements with a context." );
+	same( jQuery("p").find(".blog").get(), q("mark", "simon"), "Finding elements with a context." );
 	
 	t( "Class selector using UTF8", ".台北Táiběi", ["utf8class1"] );
 	//t( "Class selector using UTF8", ".台北", ["utf8class1","utf8class2"] );
@@ -140,15 +146,15 @@ test("class", function() {
 
 	var div = document.createElement("div");
   div.innerHTML = "<div class='test e'></div><div class='test'></div>";
-	isSet( jQuery(".e", div), [ div.firstChild ], "Finding a second class." );
+	same( jQuery(".e", div).get(), [ div.firstChild ], "Finding a second class." );
 
 	div.lastChild.className = "e";
 
-	isSet( jQuery(".e", div), [ div.firstChild, div.lastChild ], "Finding a modified class." );
+	same( jQuery(".e", div).get(), [ div.firstChild, div.lastChild ], "Finding a modified class." );
 });
 
 test("name", function() {
-	expect(11);
+	expect(14);
 
 	t( "Name selector", "input[name=action]", ["text1"] );
 	t( "Name selector with single quotes", "input[name='action']", ["text1"] );
@@ -160,29 +166,32 @@ test("name", function() {
 
 	t( "Name selector for grouped input", "input[name='types[]']", ["types_all", "types_anime", "types_movie"] )
 
-	isSet( jQuery("#form").find("input[name=action]"), q("text1"), "Name selector within the context of another element" );
-	isSet( jQuery("#form").find("input[name='foo[bar]']"), q("hidden2"), "Name selector for grouped form element within the context of another element" );
+	same( jQuery("#form").find("input[name=action]").get(), q("text1"), "Name selector within the context of another element" );
+	same( jQuery("#form").find("input[name='foo[bar]']").get(), q("hidden2"), "Name selector for grouped form element within the context of another element" );
 
-	var a = jQuery('<a id="tName1ID" name="tName1">tName1 A</a><a id="tName2ID" name="tName2">tName2 A</a><div id="tName1">tName1 Div</div>').appendTo('#main');
+	var a = jQuery('<div><a id="tName1ID" name="tName1">tName1 A</a><a id="tName2ID" name="tName2">tName2 A</a><div id="tName1">tName1 Div</div></div>').appendTo('#main').children();
+
+	equals( a.length, 3, "Make sure the right number of elements were inserted." );
+	equals( a[1].id, "tName2ID", "Make sure the right number of elements were inserted." );
 
 	t( "Find elements that have similar IDs", "[name=tName1]", ["tName1ID"] );
 	t( "Find elements that have similar IDs", "[name=tName2]", ["tName2ID"] );
+	t( "Find elements that have similar IDs", "#tName2ID", ["tName2ID"] );
 
 	a.remove();
 });
 
-
 test("multiple", function() {
 	expect(4);
 	
-	t( "Comma Support", "h2, p", ["banner","userAgent","firstp","ap","sndp","en","sap","first"]);
-	t( "Comma Support", "h2 , p", ["banner","userAgent","firstp","ap","sndp","en","sap","first"]);
-	t( "Comma Support", "h2 , p", ["banner","userAgent","firstp","ap","sndp","en","sap","first"]);
-	t( "Comma Support", "h2,p", ["banner","userAgent","firstp","ap","sndp","en","sap","first"]);
+	t( "Comma Support", "h2, p", ["qunit-banner","qunit-userAgent","firstp","ap","sndp","en","sap","first"]);
+	t( "Comma Support", "h2 , p", ["qunit-banner","qunit-userAgent","firstp","ap","sndp","en","sap","first"]);
+	t( "Comma Support", "h2 , p", ["qunit-banner","qunit-userAgent","firstp","ap","sndp","en","sap","first"]);
+	t( "Comma Support", "h2,p", ["qunit-banner","qunit-userAgent","firstp","ap","sndp","en","sap","first"]);
 });
 
 test("child and adjacent", function() {
-	expect(49);
+	expect(27);
 	t( "Child", "p > a", ["simon1","google","groups","mark","yahoo","simon"] );
 	t( "Child", "p> a", ["simon1","google","groups","mark","yahoo","simon"] );
 	t( "Child", "p >a", ["simon1","google","groups","mark","yahoo","simon"] );
@@ -199,18 +208,80 @@ test("child and adjacent", function() {
 	t( "Adjacent", "p[lang=en] + p", ["sap"] );
 	t( "Adjacent", "a.GROUPS + code + a", ["mark"] );
 	t( "Comma, Child, and Adjacent", "a + a, code > a", ["groups","anchor1","anchor2"] );
+	t( "Element Preceded By", "p ~ div", ["foo", "moretests","tabindex-tests", "liveHandlerOrder", "siblingTest"] );
+	t( "Element Preceded By", "#first ~ div", ["moretests","tabindex-tests", "liveHandlerOrder", "siblingTest"] );
+	t( "Element Preceded By", "#groups ~ a", ["mark"] );
+	t( "Element Preceded By", "#length ~ input", ["idTest"] );
+	t( "Element Preceded By", "#siblingfirst ~ em", ["siblingnext"] );
 
 	t( "Verify deep class selector", "div.blah > p > a", [] );
 
 	t( "No element deep selector", "div.foo > span > a", [] );
-	t( "No element not selector", ".container div:not(.excluded) div", [] );
 
-	isSet( jQuery("> :first", document.getElementById("nothiddendiv")), q("nothiddendivchild"), "Verify child context positional selctor" );
-	isSet( jQuery("> :eq(0)", document.getElementById("nothiddendiv")), q("nothiddendivchild"), "Verify child context positional selctor" );
-	isSet( jQuery("> *:first", document.getElementById("nothiddendiv")), q("nothiddendivchild"), "Verify child context positional selctor" );
+	same( jQuery("> :first", document.getElementById("nothiddendiv")).get(), q("nothiddendivchild"), "Verify child context positional selctor" );
+	same( jQuery("> :eq(0)", document.getElementById("nothiddendiv")).get(), q("nothiddendivchild"), "Verify child context positional selctor" );
+	same( jQuery("> *:first", document.getElementById("nothiddendiv")).get(), q("nothiddendivchild"), "Verify child context positional selctor" );
 
 	t( "Non-existant ancestors", ".fototab > .thumbnails > a", [] );
+});
+
+test("attributes", function() {
+	expect(34);
+	t( "Attribute Exists", "a[title]", ["google"] );
+	t( "Attribute Exists", "*[title]", ["google"] );
+	t( "Attribute Exists", "[title]", ["google"] );
+	t( "Attribute Exists", "a[ title ]", ["google"] );
 	
+	t( "Attribute Equals", "a[rel='bookmark']", ["simon1"] );
+	t( "Attribute Equals", 'a[rel="bookmark"]', ["simon1"] );
+	t( "Attribute Equals", "a[rel=bookmark]", ["simon1"] );
+	t( "Attribute Equals", "a[href='http://www.google.com/']", ["google"] );
+	t( "Attribute Equals", "a[ rel = 'bookmark' ]", ["simon1"] );
+
+	document.getElementById("anchor2").href = "#2";
+	t( "href Attribute", "p a[href^=#]", ["anchor2"] );
+	t( "href Attribute", "p a[href*=#]", ["simon1", "anchor2"] );
+
+	t( "for Attribute", "form label[for]", ["label-for"] );
+	t( "for Attribute in form", "#form [for=action]", ["label-for"] );
+
+	t( "Attribute containing []", "input[name^='foo[']", ["hidden2"] );
+	t( "Attribute containing []", "input[name^='foo[bar]']", ["hidden2"] );
+	t( "Attribute containing []", "input[name*='[bar]']", ["hidden2"] );
+	t( "Attribute containing []", "input[name$='bar]']", ["hidden2"] );
+	t( "Attribute containing []", "input[name$='[bar]']", ["hidden2"] );
+	t( "Attribute containing []", "input[name$='foo[bar]']", ["hidden2"] );
+	t( "Attribute containing []", "input[name*='foo[bar]']", ["hidden2"] );
+	
+	t( "Multiple Attribute Equals", "#form input[type='radio'], #form input[type='hidden']", ["radio1", "radio2", "hidden1"] );
+	t( "Multiple Attribute Equals", "#form input[type='radio'], #form input[type=\"hidden\"]", ["radio1", "radio2", "hidden1"] );
+	t( "Multiple Attribute Equals", "#form input[type='radio'], #form input[type=hidden]", ["radio1", "radio2", "hidden1"] );
+	
+	t( "Attribute selector using UTF8", "span[lang=中文]", ["台北"] );
+	
+	t( "Attribute Begins With", "a[href ^= 'http://www']", ["google","yahoo"] );
+	t( "Attribute Ends With", "a[href $= 'org/']", ["mark"] );
+	t( "Attribute Contains", "a[href *= 'google']", ["google","groups"] );
+	t( "Attribute Is Not Equal", "#ap a[hreflang!='en']", ["google","groups","anchor1"] );
+
+	t("Empty values", "#select1 option[value='']", ["option1a"]);
+	t("Empty values", "#select1 option[value!='']", ["option1b","option1c","option1d"]);
+	
+	t("Select options via :selected", "#select1 option:selected", ["option1a"] );
+	t("Select options via :selected", "#select2 option:selected", ["option2d"] );
+	t("Select options via :selected", "#select3 option:selected", ["option3b", "option3c"] );
+	
+	t( "Grouped Form Elements", "input[name='foo[bar]']", ["hidden2"] );
+});
+
+test("pseudo - child", function() {
+	expect(31);
+	t( "First Child", "p:first-child", ["firstp","sndp"] );
+	t( "Last Child", "p:last-child", ["sap"] );
+	t( "Only Child", "a:only-child", ["simon1","anchor1","yahoo","anchor2","liveLink1","liveLink2"] );
+	t( "Empty", "ul:empty", ["firstUL"] );
+	t( "Is A Parent", "p:parent", ["firstp","ap","sndp","en","sap","first"] );
+
 	t( "First Child", "p:first-child", ["firstp","sndp"] );
 	t( "Nth Child", "p:nth-child(1)", ["firstp","sndp"] );
 	t( "Not Nth Child", "p:not(:nth-child(1))", ["ap","en","sap","first"] );
@@ -249,87 +320,24 @@ test("child and adjacent", function() {
 	t( "Nth-child", "#form select:first option:nth-child(-n+3)", ["option1a", "option1b", "option1c"] );
 });
 
-test("attributes", function() {
-	expect(37);
-	t( "Attribute Exists", "a[title]", ["google"] );
-	t( "Attribute Exists", "*[title]", ["google"] );
-	t( "Attribute Exists", "[title]", ["google"] );
-	t( "Attribute Exists", "a[ title ]", ["google"] );
-	
-	t( "Attribute Equals", "a[rel='bookmark']", ["simon1"] );
-	t( "Attribute Equals", 'a[rel="bookmark"]', ["simon1"] );
-	t( "Attribute Equals", "a[rel=bookmark]", ["simon1"] );
-	t( "Attribute Equals", "a[href='http://www.google.com/']", ["google"] );
-	t( "Attribute Equals", "a[ rel = 'bookmark' ]", ["simon1"] );
+test("pseudo - misc", function() {
+	expect(6);
 
-	document.getElementById("anchor2").href = "#2";
-	t( "href Attribute", "p a[href^=#]", ["anchor2"] );
-	t( "href Attribute", "p a[href*=#]", ["simon1", "anchor2"] );
+	t( "Headers", ":header", ["qunit-header", "qunit-banner", "qunit-userAgent"] );
+	t( "Has Children - :has()", "p:has(a)", ["firstp","ap","en","sap"] );
 
-	t( "for Attribute", "form label[for]", ["label-for"] );
-	t( "for Attribute in form", "#form [for=action]", ["label-for"] );
-
-	jQuery("form input")[0].test = 0;
-	jQuery("form input")[1].test = 1;
-
-  // Disabled tests - expandos don't work in all browsers
-	//t( "Expando attribute", "form input[test]", ["text1", "text2"] );
-	//t( "Expando attribute value", "form input[test=0]", ["text1"] );
-	//t( "Expando attribute value", "form input[test=1]", ["text2"] );
-	
-	t( "Attribute containing []", "input[name^='foo[']", ["hidden2"] );
-	t( "Attribute containing []", "input[name^='foo[bar]']", ["hidden2"] );
-	t( "Attribute containing []", "input[name*='[bar]']", ["hidden2"] );
-	t( "Attribute containing []", "input[name$='bar]']", ["hidden2"] );
-	t( "Attribute containing []", "input[name$='[bar]']", ["hidden2"] );
-	t( "Attribute containing []", "input[name$='foo[bar]']", ["hidden2"] );
-	t( "Attribute containing []", "input[name*='foo[bar]']", ["hidden2"] );
-	
-	t( "Multiple Attribute Equals", "#form input[type='radio'], #form input[type='hidden']", ["radio1", "radio2", "hidden1"] );
-	t( "Multiple Attribute Equals", "#form input[type='radio'], #form input[type=\"hidden\"]", ["radio1", "radio2", "hidden1"] );
-	t( "Multiple Attribute Equals", "#form input[type='radio'], #form input[type=hidden]", ["radio1", "radio2", "hidden1"] );
-	
-	t( "Attribute selector using UTF8", "span[lang=中文]", ["台北"] );
-	
-	t( "Attribute Begins With", "a[href ^= 'http://www']", ["google","yahoo"] );
-	t( "Attribute Ends With", "a[href $= 'org/']", ["mark"] );
-	t( "Attribute Contains", "a[href *= 'google']", ["google","groups"] );
-	t( "Attribute Is Not Equal", "#ap a[hreflang!='en']", ["google","groups","anchor1"] );
-
-	t("Empty values", "#select1 option[value='']", ["option1a"]);
-	t("Empty values", "#select1 option[value!='']", ["option1b","option1c","option1d"]);
-	
-	t("Select options via :selected", "#select1 option:selected", ["option1a"] );
-	t("Select options via :selected", "#select2 option:selected", ["option2d"] );
-	t("Select options via :selected", "#select3 option:selected", ["option3b", "option3c"] );
-	
-	t( "Grouped Form Elements", "input[name='foo[bar]']", ["hidden2"] );
-	
-	t( ":not() Existing attribute", "#form select:not([multiple])", ["select1", "select2"]);
-	t( ":not() Equals attribute", "#form select:not([name=select1])", ["select2", "select3"]);
-	t( ":not() Equals quoted attribute", "#form select:not([name='select1'])", ["select2", "select3"]);
-});
-
-test("pseudo (:) selectors", function() {
-	expect(70);
-	t( "First Child", "p:first-child", ["firstp","sndp"] );
-	t( "Last Child", "p:last-child", ["sap"] );
-	t( "Only Child", "a:only-child", ["simon1","anchor1","yahoo","anchor2","liveLink1","liveLink2"] );
-	t( "Empty", "ul:empty", ["firstUL"] );
-	t( "Enabled UI Element", "#form input:not([type=hidden]):enabled", ["text1","radio1","radio2","check1","check2","hidden2","name","search"] );
-	t( "Disabled UI Element", "#form input:disabled", ["text2"] );
-	t( "Checked UI Element", "#form input:checked", ["radio2","check1"] );
-	t( "Selected Option Element", "#form option:selected", ["option1a","option2d","option3b","option3c"] );
 	t( "Text Contains", "a:contains('Google')", ["google","groups"] );
 	t( "Text Contains", "a:contains('Google Groups')", ["groups"] );
 
 	t( "Text Contains", "a:contains('Google Groups (Link)')", ["groups"] );
 	t( "Text Contains", "a:contains('(Link)')", ["groups"] );
+});
 
-	t( "Element Preceded By", "p ~ div", ["foo", "moretests","tabindex-tests", "liveHandlerOrder"] );
+
+test("pseudo - :not", function() {
+	expect(24);
 	t( "Not", "a.blog:not(.link)", ["mark"] );
 	t( "Not - multiple", "#form option:not(:contains('Nothing'),#option1b,:selected)", ["option1c", "option1d", "option2b", "option2c", "option3d", "option3e"] );
-	//t( "Not - complex", "#form option:not([id^='opt']:nth-child(-n+3))", [ "option1a", "option1d", "option2d", "option3d", "option3e"] );
 	t( "Not - recursive", "#form option:not(:not(:selected))[id^='option3']", [ "option3b", "option3c"] );
 
 	t( ":not() failing interior", "p:not(.foo)", ["firstp","ap","sndp","en","sap","first"] );
@@ -347,7 +355,20 @@ test("pseudo (:) selectors", function() {
 	t( ":not Multiple", "p:not(p,a)", [] );
 	t( ":not Multiple", "p:not(a,p,b)", [] );
 	t( ":not Multiple", ":input:not(:image,:input,:submit)", [] );
-	
+
+	t( "No element not selector", ".container div:not(.excluded) div", [] );
+
+	t( ":not() Existing attribute", "#form select:not([multiple])", ["select1", "select2"]);
+	t( ":not() Equals attribute", "#form select:not([name=select1])", ["select2", "select3"]);
+	t( ":not() Equals quoted attribute", "#form select:not([name='select1'])", ["select2", "select3"]);
+
+	t( ":not() Multiple Class", "#foo a:not(.blog)", ["yahoo","anchor2"] );
+	t( ":not() Multiple Class", "#foo a:not(.link)", ["yahoo","anchor2"] );
+	t( ":not() Multiple Class", "#foo a:not(.blog.link)", ["yahoo","anchor2"] );
+});
+
+test("pseudo - position", function() {	
+	expect(25);
 	t( "nth Element", "p:nth(1)", ["ap"] );
 	t( "First Element", "p:first", ["firstp"] );
 	t( "Last Element", "p:last", ["first"] );
@@ -356,12 +377,6 @@ test("pseudo (:) selectors", function() {
 	t( "Position Equals", "p:eq(1)", ["ap"] );
 	t( "Position Greater Than", "p:gt(0)", ["ap","sndp","en","sap","first"] );
 	t( "Position Less Than", "p:lt(3)", ["firstp","ap","sndp"] );
-	t( "Is A Parent", "p:parent", ["firstp","ap","sndp","en","sap","first"] );
-	t( "Is Visible", "#form input:visible", [] );
-	t( "Is Visible", "div:visible:not(.testrunner-toolbar):lt(2)", ["nothiddendiv", "nothiddendivchild"] );
-	t( "Is Hidden", "#form input:hidden", ["text1","text2","radio1","radio2","check1","check2","hidden1","hidden2","name","search"] );
-	t( "Is Hidden", "#main:hidden", ["main"] );
-	t( "Is Hidden", "#dl:hidden", ["dl"] );
 
 	t( "Check position filtering", "div#nothiddendiv:eq(0)", ["nothiddendiv"] );
 	t( "Check position filtering", "div#nothiddendiv:last", ["nothiddendiv"] );
@@ -375,13 +390,42 @@ test("pseudo (:) selectors", function() {
 
 	t( "Check element position", "div div:eq(0)", ["nothiddendivchild"] );
 	t( "Check element position", "div div:eq(5)", ["t2037"] );
-	t( "Check element position", "div div:eq(27)", ["hide"] );
+	t( "Check element position", "div div:eq(28)", ["hide"] );
 	t( "Check element position", "div div:first", ["nothiddendivchild"] );
 	t( "Check element position", "div > div:first", ["nothiddendivchild"] );
 	t( "Check element position", "#dl div:first div:first", ["foo"] );
 	t( "Check element position", "#dl div:first > div:first", ["foo"] );
 	t( "Check element position", "div#nothiddendiv:first > div:first", ["nothiddendivchild"] );
-	
+});
+
+if ( (window.Sizzle || jQuery.find).selectors.filters.visibility ) {
+test("pseudo - visibility", function() {
+	expect(11);
+
+	t( "Is Visible", "#form input:visible", [] );
+	t( "Is Visible", "div:visible:not(#qunit-testrunner-toolbar):lt(2)", ["nothiddendiv", "nothiddendivchild"] );
+	t( "Is Hidden", "#form input:hidden", ["text1","text2","radio1","radio2","check1","check2","hidden1","hidden2","name","search"] );
+	t( "Is Hidden", "#main:hidden", ["main"] );
+	t( "Is Hidden", "#dl:hidden", ["dl"] );
+
+	var $div = jQuery('<div/>').appendTo("body");
+	$div.css({ fontSize: 0, lineHeight: 0 });// IE also needs to set font-size and line-height to 0
+	$div.width(1).height(0);
+	t( "Is Visible", '#nothiddendivchild:visible', ['nothiddendivchild'] );
+	t( "Is Not Visible", '#nothiddendivchild:hidden', [] );
+	$div.width(0).height(1);
+	t( "Is Visible", '#nothiddendivchild:visible', ['nothiddendivchild'] );
+	t( "Is Not Visible", '#nothiddendivchild:hidden', [] );
+	$div.width(1).height(1);
+	t( "Is Visible", '#nothiddendivchild:visible', ['nothiddendivchild'] );
+	t( "Is Not Visible", '#nothiddendivchild:hidden', [] );
+	$div.remove();
+});
+}
+
+test("pseudo - form", function() {
+	expect(8);
+
 	t( "Form element :input", "#form :input", ["text1", "text2", "radio1", "radio2", "check1", "check2", "hidden1", "hidden2", "name", "search", "button", "area1", "select1", "select2", "select3"] );
 	t( "Form element :radio", "#form :radio", ["radio1", "radio2"] );
 	t( "Form element :checkbox", "#form :checkbox", ["check1", "check2"] );
@@ -390,6 +434,5 @@ test("pseudo (:) selectors", function() {
 	t( "Form element :checkbox:checked", "#form :checkbox:checked", ["check1"] );
 	t( "Form element :radio:checked, :checkbox:checked", "#form :radio:checked, #form :checkbox:checked", ["radio2", "check1"] );
 
-	t( "Headers", ":header", ["header", "banner", "userAgent"] );
-	t( "Has Children - :has()", "p:has(a)", ["firstp","ap","en","sap"] );
+	t( "Selected Option Element", "#form option:selected", ["option1a","option2d","option3b","option3c"] );
 });
